package com.dynamsoft.ipcamera;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.view.Display;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import android.os.AsyncTask;
import android.util.Log;;

import java.io.IOException;

import static com.dynamsoft.ipcamera.Defaults.CAMERA_WINDOW_HEIGHT;
import static com.dynamsoft.ipcamera.Defaults.CAMERA_WINDOW_WIDHT;
import static com.dynamsoft.ipcamera.Defaults.DEFAULT_TCP_SERVER_PORT;
import static com.dynamsoft.ipcamera.Defaults.DEFAULT_UDP_SERVER_PORT;

public class IPCamera extends Activity {
    private CameraPreview mPreview;
    private CameraManager mCameraManager;
    private boolean mIsOn = true; // Flag start socket
    private SocketClient udpThread;
    private Button mButton;
    private StringBuffer points;
    private String mIP;
	private String[] params =new String[2];
	private int udpPort = DEFAULT_UDP_SERVER_PORT;
    private String tcpPort = DEFAULT_TCP_SERVER_PORT;
    protected PositionReceiver _positionReceiver;
	static Camere GestCam;
	private GLSurfaceView mGLView;
	private boolean isDefaultStart;
	WindowManager wm;

	/*
	* 	Il metodo crea il pulsante di avvio del socket e la preview della camera da mostrare
	*
	* 	mIsOn : flag che viene posto a false dopo che è stato avviao il socket, in mododa non far
	* 	partire un altra inizializzazione.
	*
	*	La  seconda parte del codice prende i dati dalla camera e li butta sulla view
	*	dell'applicazione --> questa view verrà eliminata quando ci sarà quella proveniente dalla
	*	app che sta sviluppando il Biask.
	*/



	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
//BIASK
		mGLView = new MyGLSurfaceView(this);
		GestCam=new Camere();
		setContentView(mGLView);
		initCameraPreview();

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.ipcamera, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// TODO Auto-generated method stub
		int id = item.getItemId();
		switch (id) {
			case R.id.action_settings:
			//chiama il metodo che applica la nuova configurazione
				setting();
				break;
			case R.id.action_start_stop:

				if (mIsOn) {
					if (mIP == null) {
						udpThread = new SocketClient(getApplicationContext(),mPreview);
						isDefaultStart=true;

					}
					else {
						udpThread = new SocketClient(getApplicationContext(),mPreview, mIP, udpPort);
						params[0] = mIP;
						params[1] = tcpPort;
						isDefaultStart=false;
					}

					_positionReceiver = new PositionReceiver();
					_positionReceiver.execute(params);

					mIsOn = false;
					item.setTitle(id).setTitle("Stop");
					//mButton.setText(R.string.stop);
				}
				else {
					closeUDPSocketClient();
					closeTCPSocketClient();
					reset();
					item.setTitle(id).setTitle("Start");
				}
				break;

		}
		
		return super.onOptionsItemSelected(item);
	}
	
	private void setting() {
		LayoutInflater factory = LayoutInflater.from(this);
        final View textEntryView = factory.inflate(R.layout.server_setting, null);
        AlertDialog dialog =  new AlertDialog.Builder(IPCamera.this)
            .setIconAttribute(android.R.attr.alertDialogIcon)
            .setTitle(R.string.setting_title)
            .setView(textEntryView)
            .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {

                	EditText ipEdit = (EditText)textEntryView.findViewById(R.id.ip_edit);
                	EditText udpPortEdit = (EditText)textEntryView.findViewById(R.id.udp_port_edit);
                	EditText tcpPortEdit = (EditText)textEntryView.findViewById(R.id.tcp_port_edit);
                	mIP = ipEdit.getText().toString();
                	udpPort = Integer.parseInt(udpPortEdit.getText().toString());
					tcpPort = tcpPortEdit.getText().toString();
                	
                	Toast.makeText(IPCamera.this, "New UDP address: " + mIP + ":" + udpPort, Toast.LENGTH_SHORT).show();
					Toast.makeText(IPCamera.this, "New TCP address: " + mIP + ":" + tcpPort, Toast.LENGTH_SHORT).show();
                }
            })
            .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {

                    /* User clicked cancel so do some stuff */
                }
            })
            .create(); // fine della creazione finestra di dialogo
        dialog.show();
	}
	/*
	* Metodo chiamato quando si preme il tasto "indietro" o centrale
	*/
	@Override
    protected void onPause() {
        super.onPause();
        closeUDPSocketClient();
        mPreview.onPause();
        mCameraManager.onPause();              // release the camera immediately on pause event
        reset();

    }
	/*
		Metodo chiamato quando si clicca sul bottone start/stop e la comunicazione è attiva
		vedi if-else su mIsOn
	 */
	private void reset() {
        mIsOn = true;
	}
	/*
		Richiamo della view dopo essere stata in pausa
	 */
	@Override
	protected void onResume() {
		// TODO Auto-generated method stub
		super.onResume();
		mCameraManager.onResume();
		mPreview.setCamera(mCameraManager.getCamera());
		if(wm==null) {
			initCameraPreview();
		}
	}

	@Override
	protected void onRestart() {
		super.onRestart();
		if(wm==null)
			initCameraPreview();
	}

	@Override
	protected void onStop() {
		wm.removeViewImmediate(mPreview); // per staccare la finestra quando si spegne app
		wm=null;
		super.onStop();
	}

	//Chiude i socket prima di killare definitivamente l'applicazione
	@Override
	protected void onDestroy() {
		if(!mIsOn) {
			closeTCPSocketClient();
			closeUDPSocketClient();
		}
		super.onDestroy();
	}

	/*
            Metodo chiamato quando si clicca sul bottone start/stop e la comunicazione è attiva
            vedi if-else su mIsOn
         */
	private void closeUDPSocketClient() {
		if (udpThread == null)
			return;


		udpThread.interrupt();
        try {
			udpThread.join();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        udpThread = null;
	}

	private void closeTCPSocketClient() {
		_positionReceiver.cancel(true);
	}


	protected class PositionReceiver extends AsyncTask<String, String, String> {
		private TcpClient tcpThread = null;
		@Override
		protected String doInBackground(String... params) {

			if(isDefaultStart) { //default init
				tcpThread = new TcpClient(new TcpClient.OnMessageReceived() {
					@Override
					//here the messageReceived method is implemented
					public void messageReceived(String message) {
						//this method calls the onProgressUpdate
						publishProgress(message);


					}
				});
			}
			else{
				tcpThread = new TcpClient(new TcpClient.OnMessageReceived() {
					@Override
					public void messageReceived(String message) {
						publishProgress(message);
					}
				},params[0], params[1]);
			}
			tcpThread.run(); //Una volta inizializzati i listener fa girare il socket in background
			if(isCancelled()) tcpThread.stopClient();
			return null;
		}

		@Override
		protected void onProgressUpdate(String... values) {
			super.onProgressUpdate(values);
			//response received from server
			Log.d("test", "posizione ricevuta:  " + values[0]);
		}

	}

	private void initCameraPreview() {

		mCameraManager = new CameraManager(this);
		try {
			mPreview = new CameraPreview(this, mCameraManager.getCamera());
		} catch (IOException e){
			e.printStackTrace();
		}
		wm = (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
		Display display = wm.getDefaultDisplay();
		Point size = new Point();
		display.getSize(size);
		WindowManager.LayoutParams parameters = new WindowManager.LayoutParams(size.x/CAMERA_WINDOW_WIDHT, size.y/CAMERA_WINDOW_HEIGHT,
				WindowManager.LayoutParams.TYPE_SYSTEM_OVERLAY,
				WindowManager.LayoutParams.FLAG_WATCH_OUTSIDE_TOUCH,
				PixelFormat.TRANSLUCENT);
		parameters.gravity= Gravity.BOTTOM|Gravity.LEFT;
		wm.addView(mPreview, parameters);

		mPreview.setZOrderOnTop(true);
		mPreview.getHolder().setFormat(PixelFormat.TRANSPARENT);
	}

}


