package com.dynamsoft.ipcamera;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLES20;
import android.opengl.GLException;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.opengl.Matrix;
import android.os.SystemClock;

import java.lang.reflect.GenericSignatureFormatError;


public class MyGLRenderer implements GLSurfaceView.Renderer {

    private Triangle mTriangle;
    private Line mLine;
    float[] scratch = new float[16];//Matrix used as support
    float[] tmpscratch = new float[16];//Matrix used as support
    private final float[] mMVPMatrix = new float[16];
    private final float[] mProjectionMatrix = new float[16];
    private final float[] mViewMatrix = new float[16];
    public volatile float mCamx,mCamy,mCamz;//Camera position x,y,z
    public float getCamX() {return mCamx;  }
    public float getCamY() {
        return mCamy;
    }
    public void setCamX(float angle) {
        mCamx = angle;
    }
    public void setCamY(float angle) {
        mCamy = angle;
    }
    public void setCamZ(float zoom){mCamz+=zoom;};
    private int last_cam;//index value of the last camera added in GestCam
    private static final float color_green[]={0.63671875f, 0.76953125f, 0.22265625f, 1.0f };//green
    private static final float color_red[]={1f, 0, 0, 1.0f };//red

    public void onSurfaceCreated(GL10 unused, EGLConfig config) {
        // Set the background frame color black
        GLES20.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

        // initialize a triangle and line
        mTriangle = new Triangle();
        mLine = new Line();
        mLine.SetColor(color_red[0],color_red[1] ,color_red[2],color_red[3]);
        //set initial z position for the camera
        mCamz=7;
    }


    public void onDrawFrame(GL10 unused) {

        if (IPCamera.GestCam.Enable==2){

            float cx1, cy1, cz1, cx2=0, cy2=0, cz2=0;

            if (IPCamera.GestCam.aCam.size() != 0)//if no camera added don't refresh screen
            {
                //Redraw background color
                GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);

                last_cam = IPCamera.GestCam.aCam.size() - 1;//get index of the last camera
                mCamx = IPCamera.GestCam.aCam.get(last_cam).cx;
                mCamy = IPCamera.GestCam.aCam.get(last_cam).cy;

                //up is vector for camera orientation
                Matrix.setLookAtM(mViewMatrix, 0, mCamx, mCamy, mCamz, mCamx, mCamy, mCamz - 4,(float) Math.sin(Math.toRadians(IPCamera.GestCam.aCam.get(last_cam).ax)), (float) Math.cos(Math.toRadians(IPCamera.GestCam.aCam.get(last_cam).ax)), 0);

                //If you want fixed camera rotation
                //Matrix.setLookAtM(mViewMatrix, 0, mCamx, mCamy, mCamz, mCamx, mCamy, mCamz - 4, 0f, 1f, 0);

                // Calculate the projection and view transformation
                Matrix.multiplyMM(mMVPMatrix, 0, mProjectionMatrix, 0, mViewMatrix, 0);

                // draw lines and Triangles
                for (int i = 0; i < last_cam; i++) {//for all the cameras
                    //read data from camera
                    cx1 = IPCamera.GestCam.aCam.get(i).cx;
                    cy1 = IPCamera.GestCam.aCam.get(i).cy;
                    cz1 = IPCamera.GestCam.aCam.get(i).cz;
                    cx2 = IPCamera.GestCam.aCam.get(i + 1).cx;
                    cy2 = IPCamera.GestCam.aCam.get(i + 1).cy;
                    cz2 = IPCamera.GestCam.aCam.get(i + 1).cz;

                    System.arraycopy(mMVPMatrix, 0, tmpscratch, 0, 16);

                    //traslate and rotate "triangle"
                    Matrix.translateM(tmpscratch,0, cx1,cy1,cz1);
                    Matrix.rotateM(scratch,0,tmpscratch,0,-IPCamera.GestCam.aCam.get(i).ax,0,0,1);
                    //draw triangle
                    mTriangle.draw(scratch,color_green);

                    //print line
                    mLine.SetVerts(cx1, cy1, cz1, cx2, cy2, cz2);
                    mLine.draw(mMVPMatrix);
                    }
                //Plot last added camera
                System.arraycopy(mMVPMatrix, 0, tmpscratch, 0, 16);

                Matrix.translateM(tmpscratch, 0, cx2, cy2, cz2);
                Matrix.rotateM(scratch,0,tmpscratch,0,-IPCamera.GestCam.aCam.get(last_cam).ax,0,0,1);
                mTriangle.draw(scratch,color_red);

            }
            IPCamera.GestCam.Enable = 1;//finish to refresh screen, let TCP process to update cameras positions
        }

    }

    public void onSurfaceChanged(GL10 unused, int width, int height) {
        GLES20.glViewport(0, 0, width, height);

        float ratio = (float) width / height;

        // this projection matrix is applied to object coordinates
        Matrix.frustumM(mProjectionMatrix, 0, -ratio, ratio, -1, 1, 1, 8000);

    }

    public static int loadShader(int type, String shaderCode){

        // create a vertex shader type (GLES20.GL_VERTEX_SHADER)
        // or a fragment shader type (GLES20.GL_FRAGMENT_SHADER)
        int shader = GLES20.glCreateShader(type);

        // add the source code to the shader and compile it
        GLES20.glShaderSource(shader, shaderCode);
        GLES20.glCompileShader(shader);

        return shader;
    }
}
