package com.dynamsoft.ipcamera;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.sql.Timestamp;
import java.util.Arrays;

import android.content.Context;
import android.util.Log;
import android.util.TimeUtils;
import android.widget.Toast;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonParser;

import static com.dynamsoft.ipcamera.Defaults.DEFAULT_SERVER_IP;
import static com.dynamsoft.ipcamera.Defaults.DEFAULT_UDP_SERVER_PORT;
import static com.dynamsoft.ipcamera.Defaults.FRAME_INTERVAL;
import static com.dynamsoft.ipcamera.Defaults.PACK_SIZE;

public class SocketClient extends Thread {
	private Socket mSocket;
	private Context context;
	private CameraPreview mCameraPreview;
	private static final String TAG = "socket";
	private String mIP = DEFAULT_SERVER_IP; // Indirizzo server! Sarebbe da farglielo trovare in automatico?
	private int mPort = DEFAULT_UDP_SERVER_PORT;





	
	public SocketClient(Context context,CameraPreview preview, String ip, int port) {
		this.context = context;
	    mCameraPreview = preview;
	    mIP = ip;
	    mPort = port;
		start();
	}
	
	public SocketClient(Context context,CameraPreview preview) {
		this.context = context;
	    mCameraPreview = preview;
		start();
	}
	@Override
	public void run() {
		// TODO Auto-generated method stub
		super.run();
		
		try {
			DatagramSocket mSocket = new DatagramSocket();
			InetAddress inetAddress = InetAddress.getByName(mIP);

			byte[] JPGBytes;
			DatagramPacket datagramPacket;
			double lastCycle = System.currentTimeMillis();
			byte[] toSend = null;
			double nextCycle;
			double duration;
			double fps;
			double kbps;



			while (true) {
			try{

				JPGBytes = mCameraPreview.getFrameJPG(mCameraPreview.getImageBuffer());
				Integer totalPack = 1 + (JPGBytes.length - 1) / PACK_SIZE;
				byte[] iBuf = intToByteArray(totalPack);

				datagramPacket = new DatagramPacket(iBuf, iBuf.length, inetAddress, mPort);

				mSocket.send(datagramPacket); // Invio lunghezza pacchetto JPG

				for (int i = 0; i < totalPack; i++) {
					toSend = Arrays.copyOfRange(JPGBytes, i * PACK_SIZE, (i + 1) * PACK_SIZE);
					if (toSend.length <= PACK_SIZE) {
						mSocket.send(new DatagramPacket(toSend, PACK_SIZE, inetAddress, mPort));


					} else {
						break;
					}


				}
				try {
					Thread.sleep(FRAME_INTERVAL);
				} catch (InterruptedException e) {
					e.printStackTrace();
					break;
				}

				if (Thread.currentThread().isInterrupted())
					break;


				nextCycle = System.currentTimeMillis();
				duration = nextCycle - lastCycle;
				fps = (1 / duration) * 1000;
				kbps = PACK_SIZE * totalPack / duration / 1024 * 8;
				String timeLog = "\teffective FPS: " + fps + " \tkbps: " + kbps;
				Log.d("CONTR_FPS", timeLog);
				lastCycle = nextCycle;


			} catch(IOException e){
				e.printStackTrace();
			} catch(IllegalArgumentException e){
				e.printStackTrace();
			}
			}

		} catch (SocketException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			Log.e(TAG, e.toString());
		} catch (UnknownHostException e){
			e.printStackTrace();
			Log.e(TAG, e.toString());
		}
		finally {
			try {
				mSocket.close();
				mSocket = null;
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void close() {
		if (mSocket != null) {
			try {
				mSocket.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	private void writeToFile(String data,Context context) {
		try {
			OutputStreamWriter outputStreamWriter = new OutputStreamWriter(context.openFileOutput("LOG_IMG.txt", Context.MODE_APPEND));
			outputStreamWriter.write(data);
			outputStreamWriter.close();
		}
		catch (IOException e) {
			Log.e("Exception", "File write failed: " + e.toString());
		}
	}
	public static byte[] intToByteArray(int a)
	{
		byte[] ret = new byte[4];
		ret[0] = (byte) (a & 0xFF);
		ret[1] = (byte) ((a >> 8) & 0xFF);
		ret[2] = (byte) ((a >> 16) & 0xFF);
		ret[3] = (byte) ((a >> 24) & 0xFF);
		return ret;
	}
}
