package com.dynamsoft.ipcamera;

import android.util.Log;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;

import static com.dynamsoft.ipcamera.Defaults.DEFAULT_SERVER_IP;
import static com.dynamsoft.ipcamera.Defaults.DEFAULT_TCP_SERVER_PORT;

public class TcpClient {


    private String servIP;
    private String servPort;

    // message to send to the server
    private String mServerMessage;
    // sends message received notifications
    private OnMessageReceived mMessageListener = null;
    // while this is true, the server will continue running
    private boolean mRun = false;
    // used to send messages
    private PrintWriter mBufferOut;
    // used to read messages from the server
    private BufferedReader mBufferIn;

    /**
     * Constructor of the class. OnMessagedReceived listens for the messages received from server
     */
    public TcpClient(OnMessageReceived listener) {
        mMessageListener = listener;
        servIP = DEFAULT_SERVER_IP;
        servPort= DEFAULT_TCP_SERVER_PORT;
    }
    public TcpClient(OnMessageReceived listener, String mServerIP, String mServerPort ) {
        mMessageListener = listener;
        servIP = mServerIP;
        servPort = mServerPort;
    }

    /**
     * Sends the message entered by client to the server
     *
     * @param message text entered by client
     */
    public void sendMessage(String message) {
        if (mBufferOut != null && !mBufferOut.checkError()) {
            mBufferOut.println(message);
            mBufferOut.flush();
        }
    }

    /**
     * Close the connection and release the members
     */
    public void stopClient() {

        mRun = false;

        if (mBufferOut != null) {
            mBufferOut.flush();
            mBufferOut.close();
        }

        mMessageListener = null;
        mBufferIn = null;
        mBufferOut = null;
        mServerMessage = null;
    }

    public void run() {

        mRun = true;
        try {
            //here you must put your computer's IP address.
            InetAddress serverAddr = InetAddress.getByName(servIP);

                if(serverAddr==null){
                    Log.e("TCP","INETADRRNULL");
                }

            Log.d("TCP Client", "C: Connecting...");

            //create a socket to make the connection with the server
            Socket socket = new Socket(serverAddr, Integer.parseInt(servPort));


            try {

                //sends the message to the server
                mBufferOut = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

                //receives the message which the server sends back
               mBufferIn = new BufferedReader(new InputStreamReader(socket.getInputStream()));


                //in this while the client listens for the messages sent by the server
                while (mRun) {

                    mServerMessage = mBufferIn.readLine();
                    //Parte mia per riempire
                    if (mServerMessage != null && mMessageListener != null) {
                        //call the method messageReceived from MyActivity class


                        IPCamera.GestCam.ReadString( mServerMessage );


                        mMessageListener.messageReceived(mServerMessage);

                    }

                }

                Log.e("RESPONSE FROM SERVER", "S: Received Message: '" + mServerMessage + "'");

            } catch (Exception e) {

                Log.e("TCP", "S: Error", e);

            } finally {
                //the socket must be closed. It is not possible to reconnect to this socket
                // after it is closed, which means a new socket instance has to be created.
                socket.close();
            }

        } catch (IOException e) {

            Log.e("TCP", "C: Error", e);

        } catch (NullPointerException e){
            Log.e("TCP", "NULLP");

        }

    }

    //Declare the interface. The method messageReceived(String message) will must be implemented in the PositionReceiver
    //class at on asynckTask doInBackground
    public interface OnMessageReceived {
        public void messageReceived(String message);
    }

}